"""
API Alerts  Wrapper
~~~~~~~~~~~~~~~~~~~

A basic wrapper for the API Alerts API.

:copyright: (c) Copyright 2025 Jared Hall
:license: MIT, see LICENSE for more details.
"""

__title__ = 'apialerts-python'
__author__ = 'apialerts'
__license__ = 'MIT'
__copyright__ = 'Copyright 2025 API Alerts'
__version__ = '1.1.1'

from .apialerts import ApiAlerts
from .models.event import ApiAlertsEvent


import asyncio
from typing import Optional

from .models.event import ApiAlertsEvent
from .network.network import send_event


class _Client:
    def __init__(self) -> None:
        """
        Initialize the Client with default values.
        """
        self.api_key: Optional[str] = None
        self.debug: bool = False

    def configure(self, api_key: str = None, debug: bool = False) -> None:
        """
        Configure the Client with an API key and debug mode.

        :param api_key: The API key for authentication.
        :param debug: Enable debug mode if True.
        """
        self.api_key = api_key
        self.debug = debug

    def send(self, data: ApiAlertsEvent) -> None:
        """
        Send the alert asynchronously using the configured API key.

        :param data: The alert request data.
        """
        if not self.api_key:
            print('x (apialerts.com) Error: API Key not provided. Use configure() to set a default key, or pass the key as a parameter to the send_with_api_key function.')
            return
        self.send_with_api_key(self.api_key, data)

    async def send_async(self, data: ApiAlertsEvent) -> None:
        """
        Send the alert asynchronously using the configured API key.

        :param data: The alert request data.
        """
        if not self.api_key:
            print('x (apialerts.com) Error: API Key not provided. Use configure() to set a default key, or pass the key as a parameter to the send_with_api_key_async function.')
            return
        await self.send_with_api_key_async(self.api_key, data)

    def send_with_api_key(self, api_key: str, data: ApiAlertsEvent) -> None:
        """
        Send the alert asynchronously using a specified API key.

        :param api_key: The API key for authentication.
        :param data: The alert request data.
        """
        validations = self.validate_event(data)
        if self.debug:
            for validation in validations:
                print(validation)
        if data.message == '':
            return

        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        loop.create_task(send_event(api_key, data, self.debug))
        loop.run_until_complete(asyncio.sleep(0))

    async def send_with_api_key_async(self, api_key: str, data: ApiAlertsEvent) -> None:
        """
        Send the alert asynchronously using a specified API key.

        :param api_key: The API key for authentication.
        :param data: The alert request data.
        """
        validations = self.validate_event(data)
        if self.debug:
            for validation in validations:
                print(validation)
        if data.message == '':
            return

        await send_event(api_key, data, self.debug)

    @staticmethod
    def validate_event(event: ApiAlertsEvent) -> [str]:
        """
        Validates and updates the event request data.
        Drops properties if they are not the correct type.

        :return: An array of validation warnings if any.
        """
        validations: [str] = []

        if not isinstance(event.message, str):
            event.message = ''
            validations += ['x (apialerts.com) Validation: Message must be a string']
        if not isinstance(event.channel, str):
            event.channel = None
            validations += ['! (apialerts.com) Validation: Dropping channel from alert - channel must be a string.']
        if event.tags is not None and not isinstance(event.tags, list):
            event.tags = None
            validations += ['! (apialerts.com) Validation: Dropping tags from alert - tags must be a list of strings']
        if event.link is not None and not isinstance(event.link, str):
            event.link = None
            validations += ['! (apialerts.com) Validation: Dropping link from alert - link must be a list of strings']

        return validations


import unittest

from apialerts.apialerts import ApiAlertsEvent
from apialerts.client import _Client


class TestClient(unittest.TestCase):
    def test_client_init(self):
        client = _Client()
        self.assertIsNone(client.api_key)
        self.assertEqual(client.debug, False)

    def test_client_configure(self):
        client = _Client()
        self.assertIsNone(client.api_key)
        self.assertEqual(client.debug, False)

        client.configure("my-api-key", True)
        self.assertEqual(client.api_key, "my-api-key")
        self.assertEqual(client.debug, True)

    def test_event_initialization(self):
        event = ApiAlertsEvent(message='Test message')
        self.assertEqual(event.message, 'Test message')
        self.assertIsNone(event.channel)
        self.assertIsNone(event.tags)
        self.assertIsNone(event.link)

    def test_event_initialization_with_optional_fields(self):
        event = ApiAlertsEvent(
            message='Test message',
            channel='Test channel',
            tags=['tag1', 'tag2'],
            link='http://example.com'
        )
        self.assertEqual(event.message, 'Test message')
        self.assertEqual(event.channel, 'Test channel')
        self.assertEqual(event.tags, ['tag1', 'tag2'])
        self.assertEqual(event.link, 'http://example.com')

    def test_event_validate_valid(self):
        data = ApiAlertsEvent(
            message='Payment Received $10',
            channel='testing',
            tags=['tag1', 'tag2'],
            link='http://example.com'
        )
        result = _Client.validate_event(data)
        self.assertEqual(result, [])

    def test_event_validate_invalid_all_except_message(self):
        data = ApiAlertsEvent(
            message='Payment Received $10',
            channel=123,
            tags='tag1',
            link=77
        )
        result = _Client.validate_event(data)
        validations = [
            '! (apialerts.com) Validation: Dropping channel from alert - channel must be a string.',
            '! (apialerts.com) Validation: Dropping tags from alert - tags must be a list of strings',
            '! (apialerts.com) Validation: Dropping link from alert - link must be a list of strings'
        ]
        self.assertEqual(result, validations)

    def test_event_validate_invalid_channel(self):
        data = ApiAlertsEvent(
            message='Payment Received $10',
            channel=123,
            tags=['tag1', 'tag2'],
            link='http://example.com'
        )
        result = _Client.validate_event(data)
        validations = ['! (apialerts.com) Validation: Dropping channel from alert - channel must be a string.']
        self.assertEqual(result, validations)

    def test_event_validate_invalid_message(self):
        data = ApiAlertsEvent(
            message=123,
            channel='testing',
            tags=['tag1', 'tag2'],
            link='http://example.com'
        )
        result = _Client.validate_event(data)
        validations = ['x (apialerts.com) Validation: Message must be a string']
        self.assertEqual(result, validations)

    def test_event_validate_invalid_tags(self):
        data = ApiAlertsEvent(
            message='Payment Received $10',
            channel='testing',
            tags='tag1',
            link='http://example.com'
        )
        result = _Client.validate_event(data)
        validations = ['! (apialerts.com) Validation: Dropping tags from alert - tags must be a list of strings']
        self.assertEqual(result, validations)

    def test_event_validate_invalid_link(self):
        data = ApiAlertsEvent(
            message='Payment Received $10',
            channel='testing',
            tags=['tag1', 'tag2'],
            link=123
        )
        result = _Client.validate_event(data)
        validations = ['! (apialerts.com) Validation: Dropping link from alert - link must be a list of strings']
        self.assertEqual(result, validations)

if __name__ == '__main__':
    unittest.main()



from .client import _Client
from .models.event import ApiAlertsEvent
from typing import Optional


class ApiAlerts:
    _instance: Optional['ApiAlerts'] = None
    _client: Optional[_Client] = None

    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super(ApiAlerts, cls).__new__(cls, *args, **kwargs)
        return cls._instance

    def __init__(self) -> None:
        if not ApiAlerts._client:
            ApiAlerts._client = _Client()

    @classmethod
    def _ensure_client(cls) -> None:
        """Ensure the client is initialized."""
        if not cls._client:
            cls._client = _Client()

    @classmethod
    def configure(cls, api_key: str, debug: bool = False) -> None:
        """Configure the API client with a default API key and debug (logging) mode."""
        cls._ensure_client()
        cls._client.configure(api_key, debug)

    @classmethod
    def send(cls, data: ApiAlertsEvent) -> None:
        """Send the alert asynchronously in the background."""
        cls._ensure_client()
        cls._client.send(data)

    @classmethod
    async def send_async(cls, data: ApiAlertsEvent) -> None:
        """Send the alert asynchronously and wait for the response."""
        cls._ensure_client()
        await cls._client.send_async(data)

    @classmethod
    def send_with_api_key(cls, api_key: str, data: ApiAlertsEvent) -> None:
        """Send the alert asynchronously in the background with a different API key than configure()."""
        cls._ensure_client()
        cls._client.send_with_api_key(api_key, data)

    @classmethod
    async def send_with_api_key_async(cls, api_key: str, data: ApiAlertsEvent) -> None:
        """Send the alert asynchronously with a different API key than configure() and wait for the response."""
        cls._ensure_client()
        await cls._client.send_with_api_key_async(api_key, data)


import unittest
import re

from apialerts.constants import _BASE_URL, _X_INTEGRATION, _X_VERSION


class TestConstants(unittest.TestCase):
    def test_base_url(self):
        self.assertEqual(_BASE_URL, "https://api.apialerts.com/event")

    def test_x_integration(self):
        self.assertEqual(_X_INTEGRATION, "python")

    def test_x_version(self):
        semver_pattern = r'^\d+\.\d+\.\d+$'
        self.assertTrue(re.match(semver_pattern, _X_VERSION), "X_VERSION should be a valid semver")

if __name__ == '__main__':
    unittest.main()


from setuptools import setup
import re

def derive_version() -> str:
    with open('apialerts/__init__.py') as f:
        return re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]', f.read(), re.MULTILINE).group(1)


setup(version=derive_version())